package com.persagy.fm.department.jms.handler;

import com.google.gson.Gson;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.exception.BusinessException;
import com.persagy.fm.common.context.AppContext;
import com.persagy.fm.common.context.DefaultAppContext;
import com.persagy.fm.common.jms.MsgBaseHandler;
import com.persagy.fm.department.constant.DepJmsTypeConstants;
import com.persagy.fm.department.constant.enums.DepTypeEnum;
import com.persagy.fm.department.constant.enums.ResourceFromEnum;
import com.persagy.fm.department.jms.model.GroupCreateMsg;
import com.persagy.fm.department.jms.model.GroupUpdateMsg;
import com.persagy.fm.department.model.ConvertDepartmentTool;
import com.persagy.fm.department.model.Department;
import com.persagy.fm.department.model.dto.AddDepartmentDTO;
import com.persagy.fm.department.model.dto.QueryDepartmentDTO;
import com.persagy.fm.department.service.IDepartmentService;
import com.persagy.fm.mybatis.handler.DynamicDataSourceHandler;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.List;

/**
 * 集团消息处理类
 *
 * @author lixing
 * @version V1.0 2021/3/18 6:18 下午
 **/
@Configuration
@Slf4j
public class GroupMsgHandler{
    @Autowired
    IDepartmentService departmentService;
    @Autowired
    MsgBaseHandler msgBaseHandler;

    /**
     * 监听saas队列
     *
     * @param message 消息体字符串
     * @param channel 通道对象
     * @param msg 消息对象
     * @author lixing
     * @version V1.0 2021/3/17 9:47 上午
     */
    @RabbitListener(queues = "${dep.amqp.queue.saas}")
    public void process(String message, Channel channel, Message msg) {
        log.info("接收到运维平台集团消息：{}", message);
        try {
            // 切换数据源
            msgBaseHandler.resetDataSourceByMsg(message);
            // 获取消息类型
            String msgType = msgBaseHandler.getMsgType(message);

            Gson gson = new Gson();
            switch (msgType) {
                case DepJmsTypeConstants.GROUP_CREATE:
                    GroupCreateMsg groupCreateMsg = gson.fromJson(message, GroupCreateMsg.class);
                    createGroup(groupCreateMsg);
                    break;
                case DepJmsTypeConstants.GROUP_UPDATE:
                    GroupUpdateMsg groupUpdateMsg = gson.fromJson(message, GroupUpdateMsg.class);
                    updateGroup(groupUpdateMsg);
                    break;
                default:
                    break;
            }
            // 手动确认消息已消费
            channel.basicAck(msg.getMessageProperties().getDeliveryTag(),false);
        } catch (Exception e) {
            try {
                log.error("处理运维平台集团消息发生异常", e);
                // 将消费失败的消息移除消息队列，避免重复消费
//                channel.basicReject(msg.getMessageProperties().getDeliveryTag(),false);
//                throw new BusinessException(ResponseCode.C0001.getCode(),
//                        "部门消息消费失败，" + e.getMessage());
            } catch (Exception ex) {
//                throw new BusinessException(ResponseCode.C0001.getCode(),
//                        "部门消息从队列中移除失败，" + ex.getMessage());
            }
        }
    }

    /**
     * 更新集团
     * @param groupUpdateMsg 集团更新消息
     */
    private void updateGroup(GroupUpdateMsg groupUpdateMsg) {
        // 找到部门类型为集团的部门
        QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
        queryDepartmentDTO.setType(DepTypeEnum.GROUP.getType());
        List<Department> departments = departmentService.queryDepartmentList(queryDepartmentDTO);
        // 更新部门的名称
        if(CollectionUtils.isEmpty(departments)) {
            return;
        }
        Department department = departments.get(0);
        department.setName(groupUpdateMsg.getGroupName());
        department.setFullPath(groupUpdateMsg.getGroupName());
        departmentService.updateDepartment(department);
    }

    /**
     * 创建集团（包含幂等性处理）
     * @param groupCreateMsg 集团创建消息
     */
    private void createGroup(GroupCreateMsg groupCreateMsg) {
        /* 先查询后创建， 通过加锁的方式避免并行导致的重复创建 */
        synchronized (groupCreateMsg.getGroupCode().intern()) {
            // 给appContext赋值
            AppContext.getContext().setAccountId(groupCreateMsg.getUpdateUser());

            // 判断集团下是否已经有类型为集团的部门
            QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
            queryDepartmentDTO.setType(DepTypeEnum.GROUP.getType());
            List<Department> departments = departmentService.queryDepartmentList(queryDepartmentDTO);

            if (CollectionUtils.isEmpty(departments)) {
                Department department = new Department();
                departmentService.setDefaultValue(department);
                department.setType(DepTypeEnum.GROUP.getType());
                department.setName(groupCreateMsg.getGroupName());
                department.setFullPath(groupCreateMsg.getGroupName());

                departmentService.createDepartment(department);
            } else {
                log.error("集团下已包含类型为集团的部门，拒绝重复创建");
            }
        }
    }
}
