package com.persagy.fm.department.service.impl;

import com.google.common.collect.Lists;
import com.persagy.fm.common.constant.DefaultValueConstants;
import com.persagy.fm.common.constant.enums.BoolEnum;
import com.persagy.fm.department.model.ConvertDepartmentTool;
import com.persagy.fm.department.model.Department;
import com.persagy.fm.department.model.dto.QueryDepartmentDTO;
import com.persagy.fm.department.model.dto.QueryDepartmentTreeDTO;
import com.persagy.fm.department.model.vo.ResponseDepartmentTreeItemVO;
import com.persagy.fm.department.service.IDepTreeService;
import com.persagy.fm.department.service.IDepartmentService;
import com.persagy.fm.department.utils.DepDataUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 部门树业务层
 *
 * @author lixing
 * @version V1.0 2021/3/12 11:49 上午
 */
@Service
public class DepTreeServiceImpl implements IDepTreeService {
    @Autowired
    private IDepartmentService departmentService;

    /**
     * 初始化树节点数据
     *
     * @return map 需要展示的每一层级的部门：level -> depList
     * @author lixing
     * @version V1.0 2021/3/12 11:58 上午
     */
    private Map<Integer, List<Department>> initTreeData(QueryDepartmentTreeDTO queryDepartmentTreeDTO) {
        Integer showLevel = getShowLevel(queryDepartmentTreeDTO);
        queryDepartmentTreeDTO.setShowLevel(showLevel);
        return initDefaultTreeData(showLevel);
    }

    /**
     * 根据选中的节点判断需要加载的层级
     *
     * @param checkIdsSet 选中的节点
     * @return 需要加载的层级
     * @author lixing
     * @version V1.0 2021/4/28 10:39 上午
     */
    private int getShowLevel(Set<String> checkIdsSet) {
        int result = getDefaultShowLevel();

        if (CollectionUtils.isEmpty(checkIdsSet)) {
            return result;
        }
        QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
        queryDepartmentDTO.setIds(new ArrayList<>(checkIdsSet));
        List<Department> departments = departmentService.queryDepartmentList(queryDepartmentDTO);


        for (Department department : departments) {
            String fullPath = department.getFullPath();
            String separator = "-";
            String[] paths = fullPath.split(separator);
            result = Math.max(paths.length, result);
        }

        return result;
    }

    /**
     * 获取前端传入的需要展示的层级，如果没有传入默认展示为2层
     *
     * @param queryDepartmentTreeDTO 部门树查询条件
     * @return 需要展示的层级
     */
    private Integer getShowLevel(QueryDepartmentTreeDTO queryDepartmentTreeDTO) {
        // 如果查询条件中有选中的节点，先判断选中节点的展示等级
        List<String> checkedIds = queryDepartmentTreeDTO.getCheckedIds();
        if (checkedIds == null) {
            checkedIds = Lists.newArrayList();
        }
        Set<String> checkIdsSet = new HashSet<>(checkedIds);
        int showLevelFromCheckedIds = getShowLevel(checkIdsSet);

        Integer showLevel = queryDepartmentTreeDTO.getShowLevel();
        if (showLevel == null) {
            showLevel = showLevelFromCheckedIds;
        } else {
            showLevel = Math.max(showLevelFromCheckedIds, showLevel);
        }
        return showLevel;
    }

    /**
     * 获取默认展示的层级数
     *
     * @return 默认展示的层级数
     * @author lixing
     * @version V1.0 2021/3/26 10:36 上午
     */
    private int getDefaultShowLevel() {
        return 2;
    }

    /**
     * 初始化默认树
     *
     * @return 需要展示的每一层级的部门：level -> depList
     * @author lixing
     * @version V1.0 2021/3/12 12:00 下午
     */
    private Map<Integer, List<Department>> initDefaultTreeData(Integer showLevel) {
        Map<Integer, List<Department>> levelDepListMap = new HashMap<>(showLevel);

        // TODO: 2021/3/9 根据当前登录人获取根节点
        // 部门树根节点的父节点id
        String rootParentId = DefaultValueConstants.NO_PARENT_ID;
        List<String> rootDeps = Lists.newArrayList(rootParentId);

        return initDefaultTreeData(levelDepListMap, 0, rootDeps, showLevel);
    }

    /**
     * 初始化一层树节点，递归方法，外部调用入参为（0， null）
     *
     * @param levelDepListMap level -> depList
     * @param level           当前层级
     * @param parentDepIdList 上一层部门Id列表
     * @param showLevel       前端传入的要展示的层级
     * @return 需要展示的每一层级的部门：level -> depList
     * @author lixing
     * @version V1.0 2021/3/12 12:02 下午
     */
    public Map<Integer, List<Department>> initDefaultTreeData(
            Map<Integer, List<Department>> levelDepListMap,
            int level,
            List<String> parentDepIdList,
            int showLevel) {
        if (CollectionUtils.isEmpty(parentDepIdList)) {
            return levelDepListMap;
        }
        levelDepListMap.put(level, departmentService.queryDepListByParentIds(parentDepIdList));

        if (level < showLevel) {
            List<Department> departments = levelDepListMap.get(level);
            departments = departments == null ? Lists.newArrayList() : departments;
            List<String> depIds = departments.stream().map(Department::getId).collect(Collectors.toList());
            initDefaultTreeData(levelDepListMap, level + 1, depIds, showLevel);
        }

        return levelDepListMap;
    }

    /**
     * 获取节点的子节点，追加到部门树
     *
     * @param levelParentIdMap level -> { parentId -> depList }
     * @param treeItem         节点
     * @param level            节点的下一级，例如根节点的下一级为 1
     * @param showLevel        前端传入的需要展示的层级
     * @author lixing
     * @version V1.0 2021/3/12 3:12 下午
     */
    public void setChildren(
            Map<Integer, Map<String, List<Department>>> levelParentIdMap,
            ResponseDepartmentTreeItemVO treeItem,
            int level,
            int showLevel) {
        Map<String, List<Department>> parentIdMap = levelParentIdMap.get(level);
        parentIdMap = parentIdMap != null ? parentIdMap : new HashMap<>(0);
        List<Department> departments = parentIdMap.get(treeItem.getId());
        List<ResponseDepartmentTreeItemVO> children = ConvertDepartmentTool.INSTANCE.convert2Tree(departments);

        if (CollectionUtils.isEmpty(children)) {
            treeItem.setHasChildren(BoolEnum.FALSE.getType());
        } else {
            treeItem.setHasChildren(BoolEnum.TRUE.getType());
        }

        // 如果目前显示等级已经大于预设的显示等级，或者当前节点没有子节点了，就停止递归
        if (level >= showLevel || CollectionUtils.isEmpty(children)) {
            return;
        }

        treeItem.setChildren(children);
        for (ResponseDepartmentTreeItemVO child : children) {
            setChildren(levelParentIdMap, child, level + 1, showLevel);
        }
    }

    /**
     * 查询部门树
     *
     * @author lixing
     * @version V1.0 2021/3/12 11:57 上午
     */
    @Override
    public List<ResponseDepartmentTreeItemVO> queryDepartmentTree(
            QueryDepartmentTreeDTO queryDepartmentTreeDTO) {

        Map<Integer, List<Department>> levelDeptListMap = initTreeData(queryDepartmentTreeDTO);

        /* checkedId为空，按查询条件中的展示层数要求加载数据
         * checkedId不为空，如果选中节点所在层级小于查询条件中的展示层数，按查询条件中的展示层数要求加载数据；
         * 否则，展示到选中的节点所在层级
         */
        if (CollectionUtils.isEmpty(levelDeptListMap)) {
            return Lists.newArrayList();
        }
        List<Department> rootList = levelDeptListMap.get(0);
        if (CollectionUtils.isEmpty(rootList)) {
            return Lists.newArrayList();
        }

        // levelDeptListMap 转换为 levelParentIdMap
        Map<Integer, Map<String, List<Department>>> levelParentIdMap = covert2LevelParentIdMap(levelDeptListMap);

        List<ResponseDepartmentTreeItemVO> result = ConvertDepartmentTool.INSTANCE.convert2Tree(rootList);
        result.forEach(treeItem -> {
            setChildren(levelParentIdMap, treeItem, 1, getShowLevel(queryDepartmentTreeDTO));
        });

        return result;
    }

    /**
     * level -> depList 转换为 level -> { parentId -> deptList }
     *
     * @param levelDeptListMap level -> depList
     * @author lixing
     * @version V1.0 2021/3/12 2:43 下午
     */
    private Map<Integer, Map<String, List<Department>>> covert2LevelParentIdMap(
            Map<Integer, List<Department>> levelDeptListMap) {
        Map<Integer, Map<String, List<Department>>> levelParentIdMap = new HashMap<>(levelDeptListMap.size());
        levelDeptListMap.forEach((level, depList) -> {
            Map<String, List<Department>> parentIdMap = DepDataUtil.sonDeps2ParentIdMap(depList);
            levelParentIdMap.put(level, parentIdMap);
        });
        return levelParentIdMap;
    }

}
