package com.persagy.fm.person.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.fm.common.model.vo.SimpleObjVO;
import com.persagy.fm.person.dao.PersonMapper;
import com.persagy.fm.person.model.Person;
import com.persagy.fm.person.model.dto.AddPerson4FrontEndDTO;
import com.persagy.fm.person.model.dto.UpdatePerson4FrontEndDTO;
import com.persagy.fm.person.service.IPersonCheckService;
import com.persagy.fm.person.service.IPersonQueryService;
import com.persagy.fm.saas.account.model.dto.PageQuerySaasAccountDTO;
import com.persagy.fm.saas.account.model.vo.SaasAccountListItemVO;
import com.persagy.fm.saas.account.service.ISaasAccountService;
import com.persagy.fm.saas.accountrole.model.vo.AccountRoleVO;
import com.persagy.fm.saas.accountrole.service.ISaasAccountRoleService;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 人员校验工具类
 *
 * @author lixing
 * @version V1.0 2021/4/29 5:27 下午
 */
@Component
public class PersonCheckServiceImpl extends ServiceImpl<PersonMapper, Person>
        implements IPersonCheckService {
    @Autowired
    private ISaasAccountService saasAccountService;
    @Autowired
    private ISaasAccountRoleService saasAccountRoleService;
    @Autowired
    private IPersonQueryService personQueryService;

    @Override
    public void checkLeader(String leaderId, Person person) {
        if (StringUtils.isBlank(leaderId)) {
            return;
        }
        String personId = person.getId();
        if (personId.equals(leaderId)) {
            throw new IllegalArgumentException("不能选择自己作为直接上级");
        }
        // 不能选择直接上级是自己的员工作为上级
        List<Person> people = personQueryService.queryByLeader(Long.valueOf(personId));
        people.forEach(tmpPerson -> {
            String tmpPersonId = tmpPerson.getId();
            if (tmpPersonId.equals(leaderId)) {
                throw new IllegalArgumentException("不能选择自己的下级作为直接上级");
            }
        });
    }

    @Override
    public void checkMainAndOtherDutiesWhenBatchUpdate(String mainDuty, List<String> otherDuties, String personId) {
        Person person = getById(personId);
        String accountId = person.getAccountId();
        // 设置人员岗位角色信息
        Map<String, AccountRoleVO> accountRoleVoMap = saasAccountRoleService.
                queryAccountRoleVoMapByAccountIdList(Lists.newArrayList(accountId));
        if (accountRoleVoMap != null) {
            AccountRoleVO accountRoleVO = accountRoleVoMap.get(accountId);
            String personMainDuty = mainDuty;
            if (mainDuty == null) {
                SimpleObjVO mainDutyVO = accountRoleVO.getMainDuty();
                personMainDuty = mainDutyVO == null ? null : mainDutyVO.getId();
            }
            List<String> personOtherDuties = otherDuties;
            if (otherDuties == null) {
                List<SimpleObjVO> otherDutiesVO = accountRoleVO.getOtherDuties();
                if (otherDutiesVO != null) {
                    personOtherDuties = otherDutiesVO.stream().
                            map(SimpleObjVO::getId).collect(Collectors.toList());
                }
            }
            if (!CollectionUtils.isEmpty(personOtherDuties)
                    && StringUtils.isNotBlank(personMainDuty)
                    && personOtherDuties.contains(personMainDuty)) {
                throw new IllegalArgumentException("人员主岗与副岗不能出现重合");
            }
        }
    }

    /**
     * 检查账号有没有被使用
     *
     * @param username 账号
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/3/23 3:15 下午
     */
    @Override
    public Boolean checkDuplicateUserName(String username) {
        if (StringUtils.isBlank(username)) {
            return false;
        }
        List<Person> people = personQueryService.queryByUsername(username);
        if (!CollectionUtils.isEmpty(people)) {
            return true;
        }

        PageQuerySaasAccountDTO pageQuerySaasAccountDTO = new PageQuerySaasAccountDTO();
        pageQuerySaasAccountDTO.setUsername(username);
        List<SaasAccountListItemVO> accountListItemVoList = saasAccountService.querySaasAccountList(pageQuerySaasAccountDTO);
        return !CollectionUtils.isEmpty(accountListItemVoList);
    }

    @Override
    public Boolean checkDuplicateMail(String mail) {
        if (StringUtils.isBlank(mail)) {
            return false;
        }
        PageQuerySaasAccountDTO pageQuerySaasAccountDTO = new PageQuerySaasAccountDTO();
        pageQuerySaasAccountDTO.setMail(mail);
        List<SaasAccountListItemVO> accountListItemVoList = saasAccountService.querySaasAccountList(pageQuerySaasAccountDTO);
        return !CollectionUtils.isEmpty(accountListItemVoList);
    }

    /**
     * 检查手机号有没有被使用
     *
     * @param phoneNum 手机号
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/3/23 3:15 下午
     */
    @Override
    public Boolean checkDuplicatePhoneNum(String phoneNum) {
        if (StringUtils.isBlank(phoneNum)) {
            return false;
        }
        PageQuerySaasAccountDTO pageQuerySaasAccountDTO = new PageQuerySaasAccountDTO();
        pageQuerySaasAccountDTO.setPhoneNum(phoneNum);
        List<SaasAccountListItemVO> accountListItemVoList = saasAccountService.querySaasAccountList(pageQuerySaasAccountDTO);
        return !CollectionUtils.isEmpty(accountListItemVoList);
    }


    /**
     * 校验身份证号重复
     *
     * @param idNumber 身份证号
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/1 3:03 下午
     */
    @Override
    public Boolean checkDuplicateIdNumber(String idNumber) {
        if (StringUtils.isBlank(idNumber)) {
            return false;
        }
        List<Person> people = personQueryService.queryByIdCard(idNumber);
        return !CollectionUtils.isEmpty(people);
    }

    /**
     * 校验工号重复
     *
     * @param jobNumber 工号
     * @return 是否重复
     * @author lixing
     * @version V1.0 2021/4/1 3:03 下午
     */
    @Override
    public Boolean checkDuplicateJobNumber(String jobNumber) {
        if (StringUtils.isBlank(jobNumber)) {
            return false;
        }
        List<Person> people = personQueryService.queryByJobNumber(jobNumber);
        return !CollectionUtils.isEmpty(people);
    }

    /**
     * 校验人员是否可删除
     *
     * @param id 人员主键
     * @return 人员do类
     * @author lixing
     * @version V1.0 2021-03-15 17:20:31
     */
    public Person checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除Person时发生异常，主键为空");
        }

        Person person = getById(id);

        if (person == null) {
            throw new IllegalArgumentException("删除Person时发生异常，找不到要删除的数据，id:" + id);
        }

        return person;
    }

    /**
     * 校验重复项
     *
     * @param addPerson4FrontEndDTO 创建人员入参
     * @author lixing
     * @version V1.0 2021/3/22 9:53 上午
     */
    public void checkAddPersonDTO(AddPerson4FrontEndDTO addPerson4FrontEndDTO) {
        // 校验身份证号
        if (checkDuplicateIdNumber(addPerson4FrontEndDTO.getIdNumber())) {
            throw new IllegalArgumentException("身份证号重复");
        }
        // 校验账号
        if (checkDuplicateUserName(addPerson4FrontEndDTO.getUsername())) {
            throw new IllegalArgumentException("账号已被使用");
        }
        // 校验手机号
        if (checkDuplicatePhoneNum(addPerson4FrontEndDTO.getPhoneNum())) {
            throw new IllegalArgumentException("手机号已被使用");
        }
        // 校验工号
        if (checkDuplicateJobNumber(addPerson4FrontEndDTO.getJobNumber())) {
            throw new IllegalArgumentException("工号已被使用");
        }
        // 校验邮箱
        if (checkDuplicateMail(addPerson4FrontEndDTO.getMail())) {
            throw new IllegalArgumentException("邮箱已被使用");
        }
    }

    /**
     * 校验重复项
     *
     * @param updatePerson4FrontEndDTO 更新人员入参
     * @author lixing
     * @version V1.0 2021/3/22 9:53 上午
     */
    public void checkUpdatePersonDTO(UpdatePerson4FrontEndDTO updatePerson4FrontEndDTO) {
        String personId = updatePerson4FrontEndDTO.getId();
        Person person = getById(personId);
        // 校验身份证号
        if (checkDuplicateIdNumber(updatePerson4FrontEndDTO.getIdNumber(), person)) {
            throw new IllegalArgumentException("身份证号重复");
        }
        // 校验账号
        if (checkDuplicateUserName(updatePerson4FrontEndDTO.getUsername(), person)) {
            throw new IllegalArgumentException("账号已被使用");
        }
        // 校验手机号
        if (checkDuplicatePhoneNum(updatePerson4FrontEndDTO.getPhoneNum(), person)) {
            throw new IllegalArgumentException("手机号已被使用");
        }
        // 校验工号
        if (checkDuplicateJobNumber(updatePerson4FrontEndDTO.getJobNumber(), person)) {
            throw new IllegalArgumentException("工号已被使用");
        }
        // 校验邮箱
        if (checkDuplicateMail(updatePerson4FrontEndDTO.getMail(), person)) {
            throw new IllegalArgumentException("邮箱已被使用");
        }
    }

    @Override
    public Boolean checkDuplicatePhoneNum(String phoneNumber, Person person) {
        if (StringUtils.isBlank(phoneNumber)) {
            return false;
        }
        if (person == null) {
            return checkDuplicatePhoneNum(phoneNumber);
        }
        SaasAccountListItemVO saasAccountListItemVO = saasAccountService.querySaasAccountDetail(person.getAccountId());
        String accountPhoneNumber = saasAccountListItemVO.getPhoneNum();
        if (phoneNumber.equals(accountPhoneNumber)) {
            return false;
        } else {
            return checkDuplicatePhoneNum(phoneNumber);
        }
    }

    @Override
    public Boolean checkDuplicateIdNumber(String idNumber, Person person) {
        if (StringUtils.isBlank(idNumber)) {
            return false;
        }
        if (person == null) {
            return checkDuplicateIdNumber(idNumber);
        }
        String personIdNumber = person.getIdNumber();
        if (idNumber.equals(personIdNumber)) {
            return false;
        } else {
            return checkDuplicateIdNumber(idNumber);
        }
    }

    @Override
    public Boolean checkDuplicateUserName(String username, Person person) {
        if (StringUtils.isBlank(username)) {
            return false;
        }
        if (person == null) {
            return checkDuplicateUserName(username);
        }
        String personUsername = person.getUsername();
        if (username.equals(personUsername)) {
            return false;
        } else {
            return checkDuplicateUserName(username);
        }
    }

    @Override
    public Boolean checkDuplicateJobNumber(String jobNumber, Person person) {
        if (StringUtils.isBlank(jobNumber)) {
            return false;
        }
        if (person == null) {
            return checkDuplicateJobNumber(jobNumber);
        }
        String personJobNumber = person.getJobNumber();
        if (jobNumber.equals(personJobNumber)) {
            return false;
        } else {
            return checkDuplicateJobNumber(jobNumber);
        }
    }

    @Override
    public Boolean checkDuplicateMail(String mail, Person person) {
        if (StringUtils.isBlank(mail)) {
            return false;
        }
        if (person == null) {
            return checkDuplicateMail(mail);
        }
        SaasAccountListItemVO saasAccountListItemVO = saasAccountService.querySaasAccountDetail(person.getAccountId());
        String accountMail = saasAccountListItemVO.getMail();
        if (mail.equals(accountMail)) {
            return false;
        } else {
            return checkDuplicateMail(mail);
        }
    }
}