package com.persagy.fm.workresume.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.fm.common.constant.enums.BoolEnum;
import com.persagy.fm.common.constant.enums.ValidEnum;
import com.persagy.fm.common.context.DefaultAppContext;
import com.persagy.fm.workresume.constant.enums.WorkResumeEnableStateEnum;
import com.persagy.fm.workresume.constant.enums.WorkResumeHiredStateEnum;
import com.persagy.fm.workresume.dao.WorkResumeMapper;
import com.persagy.fm.workresume.model.ConvertWorkResumeTool;
import com.persagy.fm.workresume.model.WorkResume;
import com.persagy.fm.workresume.model.dto.AddWorkResumeDTO;
import com.persagy.fm.workresume.model.dto.QueryWorkResumeDTO;
import com.persagy.fm.workresume.model.dto.UpdateWorkResumeDTO;
import com.persagy.fm.workresume.service.IWorkResumeService;
import org.assertj.core.util.Lists;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 工作信息(WorkResume) service层
 *
 * @author lixing
 * @version V1.0 2021-03-15 17:20:06
 */
@Service
public class WorkResumeServiceImpl extends ServiceImpl<WorkResumeMapper, WorkResume>
        implements IWorkResumeService {

    /**
     * 创建工作信息
     *
     * @return Long 工作信息主键
     * @author lixing
     * @version V1.0 2021-03-15 17:20:06
     */
    @Override
    public String createWorkResume(AddWorkResumeDTO addWorkResumeDTO) {
        WorkResume workResume = ConvertWorkResumeTool.INSTANCE.convertAddDto2Entity(addWorkResumeDTO);
        // 设置默认值
        setDefaultValue(workResume);
        save(workResume);
        return workResume.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param workResume 工作信息实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(WorkResume workResume) {
        workResume.setCreator(DefaultAppContext.getContext().getAccountId());
        if (workResume.getHiredDate() == null) {
            workResume.setHiredDate(new Date());
        }
        if (workResume.getHiredDate() == null) {
            workResume.setHiredState(WorkResumeHiredStateEnum.ON_JOB.getType());
        }
        if (workResume.getEnableState() == null) {
            workResume.setEnableState(WorkResumeEnableStateEnum.ENABLE.getType());
        }
    }

    /**
     * 工作信息详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-03-15 17:20:06
     */
    @Override
    public WorkResume queryWorkResumeDetail(String id) {
        WorkResume workResume = getById(id);
        if (workResume == null) {
            throw new IllegalArgumentException("查看WorkResume详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return workResume;
    }

    /**
     * 更新工作信息
     *
     * @author lixing
     * @version V1.0 2021-03-15 17:20:06
     */
    @Override
    public void updateWorkResume(UpdateWorkResumeDTO updateWorkResumeDTO) {
        WorkResume workResume = getById(updateWorkResumeDTO.getId());
        workResume = ConvertWorkResumeTool.INSTANCE.convertUpdateDto2Entity(workResume, updateWorkResumeDTO);
        workResume.setModifier(DefaultAppContext.getContext().getAccountId());
        updateById(workResume);
    }

    /**
     * 校验工作信息是否可删除
     *
     * @param id 工作信息主键
     * @return 工作信息do类
     * @author lixing
     * @version V1.0 2021-03-15 17:20:06
     */
    public WorkResume checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除WorkResume时发生异常，主键为空");
        }

        WorkResume workResume = getById(id);

        if (workResume == null) {
            throw new IllegalArgumentException("删除WorkResume时发生异常，找不到要删除的数据，id:" + id);
        }

        return workResume;
    }

    /**
     * 删除工作信息
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-03-15 17:20:06
     */
    @Override
    public void deleteWorkResume(String id) {
        // 校验是否可删除
        WorkResume workResume = checkDeletable(id);

        workResume.setValid(ValidEnum.FALSE.getType());
        updateById(workResume);
    }

    /**
     * 查询工作信息
     *
     * @return List<WorkResume>
     * @author lixing
     * @version V1.0 2021-03-15 17:20:06
     */
    @Override
    public List<WorkResume> queryWorkResumeList(QueryWorkResumeDTO queryWorkResumeDTO) {
        QueryWrapper<WorkResume> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(WorkResume.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, WorkResume.PROP_CREATIONTIME);

        if (queryWorkResumeDTO != null) {

            if (!StringUtils.isEmpty(queryWorkResumeDTO.getPersonId())) {
                queryWrapper.eq(WorkResume.PROP_PERSON_ID, queryWorkResumeDTO.getPersonId());
            }

            if (!CollectionUtils.isEmpty(queryWorkResumeDTO.getPersonIds())) {
                queryWrapper.in(WorkResume.PROP_PERSON_ID, queryWorkResumeDTO.getPersonIds());
            }

            if (!StringUtils.isEmpty(queryWorkResumeDTO.getDepId())) {
                queryWrapper.eq(WorkResume.PROP_DEP_ID, queryWorkResumeDTO.getDepId());
            }

            if (!CollectionUtils.isEmpty(queryWorkResumeDTO.getDepIds())) {
                queryWrapper.in(WorkResume.PROP_DEP_ID, queryWorkResumeDTO.getDepIds());
            }

            if (!StringUtils.isEmpty(queryWorkResumeDTO.getIsLeader())) {
                queryWrapper.eq(WorkResume.PROP_IS_LEADER, queryWorkResumeDTO.getIsLeader());
            }

            if (!StringUtils.isEmpty(queryWorkResumeDTO.getHiredState())) {
                queryWrapper.eq(WorkResume.PROP_HIRED_STATE, queryWorkResumeDTO.getHiredState());
            }

            if (!StringUtils.isEmpty(queryWorkResumeDTO.getEnableState())) {
                queryWrapper.eq(WorkResume.PROP_ENABLE_STATE, queryWorkResumeDTO.getEnableState());
            }

            if (queryWorkResumeDTO.getHiredDate() != null) {
                queryWrapper.eq(WorkResume.PROP_HIRED_DATE, queryWorkResumeDTO.getHiredDate());
            }

            if (queryWorkResumeDTO.getDepartureDate() != null) {
                queryWrapper.eq(WorkResume.PROP_DEPARTURE_DATE, queryWorkResumeDTO.getDepartureDate());
            }

        }

        return list(queryWrapper);
    }

    @Override
    public void batchUpdateWorkResume(List<WorkResume> workResumes) {
        updateBatchById(workResumes);
    }

    @Override
    public List<WorkResume> queryWorkResumeListByPersonIdList(List<String> personIdList) {
        QueryWrapper<WorkResume> workResumeQueryWrapper = new QueryWrapper<>();
        workResumeQueryWrapper.in(WorkResume.PROP_PERSON_ID, personIdList);
        workResumeQueryWrapper.eq(WorkResume.PROP_VALID, ValidEnum.TRUE.getType());
        return list(workResumeQueryWrapper);
    }

    @Override
    public void changeWorkResumeHiredStateByPersonId(String personId, String workResumeHiredState) {
        // 校验更新人员工作信息在职状态入参
        if (StringUtils.isEmpty(personId)) {
            throw new IllegalArgumentException("更新人员工作信息状态，缺少personId");
        }

        QueryWorkResumeDTO queryWorkResumeDTO = new QueryWorkResumeDTO();
        queryWorkResumeDTO.setPersonId(personId);

        List<WorkResume> workResumes = queryWorkResumeList(queryWorkResumeDTO);
        workResumes.forEach(workResume -> workResume.setHiredState(workResumeHiredState));
        updateBatchById(workResumes);
    }

    @Override
    public void changeWorkResumeEnableStateByPersonId(String personId, String workResumeEnableState) {
        // 校验更新人员工作信息启用状态入参
        if (StringUtils.isEmpty(personId)) {
            throw new IllegalArgumentException("更新人员工作信息状态，缺少personId");
        }

        QueryWorkResumeDTO queryWorkResumeDTO = new QueryWorkResumeDTO();
        queryWorkResumeDTO.setPersonId(personId);

        List<WorkResume> workResumes = queryWorkResumeList(queryWorkResumeDTO);
        workResumes.forEach(workResume -> workResume.setEnableState(workResumeEnableState));
        updateBatchById(workResumes);
    }

    @Override
    public List<String> queryDepManagers(String depId) {
        if (StringUtils.isEmpty(depId)) {
            throw new IllegalArgumentException("查询部门负责人，部门id不能为空");
        }
        QueryWorkResumeDTO queryWorkResumeDTO = new QueryWorkResumeDTO();
        queryWorkResumeDTO.setDepId(depId);
        queryWorkResumeDTO.setIsLeader(BoolEnum.TRUE.getType());
        List<WorkResume> workResumes = queryWorkResumeList(queryWorkResumeDTO);
        List<String> managerIds = Lists.newArrayList();
        if (!CollectionUtils.isEmpty(workResumes)) {
            managerIds = workResumes.stream().map(WorkResume::getPersonId).collect(Collectors.toList());
        }
        return managerIds;
    }

    @Override
    public void updatePersonWorkResume(String personId, List<String> addedDepIds, List<String> deletedDepIds) {
        // 添加工作信息
        List<WorkResume> addedWorkResumes = Lists.newArrayList();
        if (!CollectionUtils.isEmpty(addedDepIds)) {
            addedDepIds.forEach(addedDepId -> {
                WorkResume workResume = new WorkResume();
                workResume.setPersonId(personId);
                workResume.setDepId(addedDepId);
                setDefaultValue(workResume);
                addedWorkResumes.add(workResume);
            });
        }
        if (!CollectionUtils.isEmpty(addedWorkResumes)) {
            saveBatch(addedWorkResumes);
        }

        // 删除工作信息
        List<WorkResume> deletedWorkResumes = Lists.newArrayList();
        if (!CollectionUtils.isEmpty(deletedDepIds)) {
            QueryWorkResumeDTO queryWorkResumeDTO = new QueryWorkResumeDTO();
            queryWorkResumeDTO.setPersonId(personId);
            queryWorkResumeDTO.setDepIds(deletedDepIds);
            deletedWorkResumes = queryWorkResumeList(queryWorkResumeDTO);
        }
        if (!CollectionUtils.isEmpty(deletedWorkResumes)) {
            deletedWorkResumes.forEach(workResume -> workResume.setValid(ValidEnum.FALSE.getType()));
            updateBatchById(deletedWorkResumes);
        }
    }
}
